/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 * 
 * Copyright (C) 2004-5. 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 * 
 * cresques@gmail.com
 */
package org.cresques.io;

import java.awt.Image;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

import org.cresques.cts.ICoordTrans;
import org.cresques.io.data.BandList;
import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Metadata;
import org.cresques.io.exceptions.NotSupportedExtensionException;
import org.cresques.io.exceptions.SupersamplingNotSupportedException;
import org.cresques.px.Extent;


/**
 * Driver to read data from memory. This can be created with a RasterBuf 
 * and its bounding box.
 * @author Nacho Brodin (nachobrodin@gmail.com)
 *
 */
public class MemoryRasterDriver extends GeoRasterFile {

	protected Extent v = null;
	protected RasterBuf buf = null;
	
	public MemoryRasterDriver(){
		super(null, null);
	}
	
	/**
	 * Constructor
	 * @param buf Data buffer
	 * @param ext bounding box
	 * @throws NotSupportedExtensionException
	 */
	public MemoryRasterDriver(RasterBuf buf, Extent ext)throws NotSupportedExtensionException{
		super(null, null);
		this.extent = this.requestExtent = ext;
		this.buf = buf;
		this.name = "";
		load();
		bandCount = buf.getBandCount(); 
		if ( bandCount > 2) {
			setBand(RED_BAND,   0);
			setBand(GREEN_BAND, 1);
			setBand(BLUE_BAND,  2);
		} else
			setBand(RED_BAND|GREEN_BAND|BLUE_BAND, 0);

		//Obtenemos el tipo de dato de gdal y lo convertimos el de RasterBuf
		setDataType(org.cresques.util.Utilities.getRasterBufTypeFromGdalType(buf.getDataType()));
	}
	
	/**
	 * 
	 */
	public GeoFile load() {
		return this;
	}
	
	/**
	 * 
	 */
	public void close() {
		
	}
	
	/**
	 * Asigna a cada banda R,G o B una banda de la imagen
	 */
	public void setBand(int flag, int bandNr) {
		super.setBand(flag, bandNr);
	}
	
	/**
	 * Asigna el extent de la vista actual. 
	 */
	public void setView(Extent e) { 
		v = new Extent(e.minX(), e.minY(), e.maxX(), e.maxY());	
	}
		
	 /**
	 * Calcula la transformacin que se produce sobre la vista cuando la imagen tiene un fichero .rmf
	 * asociado. En Gdal el origen de coordenadas en Y es el valor mnimo y crece hasta el mximo. De la
	 * misma forma calcula la matriz de transformacin de la cabecera del fichero o del world file asociado
	 * @param originX Origen de la imagen en la coordenada X
	 * @param originY Origen de la imagen en la coordenada Y
	 */
	public void setExtentTransform(double originX, double originY, double psX, double psY) {		
		transformRMF.setToTranslation(originX, originY);
		transformRMF.scale(psX, psY);
	}
	
	/**
	 * Obtiene extent de la vista actual
	 */
	public Extent getView() { 
		return v; 
	}
	
	/**
	 * Obtiene la anchura del fichero
	 */
	public int getWidth() {	
		return buf.getWidth(); 
	}
	
	/**
	 * Obtiene la altura del fichero
	 */
	public int getHeight() { 
		return buf.getHeight();
	}

	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#reProject(org.cresques.cts.ICoordTrans)
	 */
	public void reProject(ICoordTrans rp) {
		// TODO Auto-generated method stub
	}
	
	/**
	 * Obtiene la orientacin de la imagen a partir del signo del tamao de pixel para poder
	 * asignarlo en el setView. Esto es util para poder conocer como debe leerse la image, 
	 * de abajo a arriba, de arriba a abajo, de izquierda a derecha o de derecha a izquierda. 
	 * La posicin habitual es la que el pixel size en X es positivo y en Y negativo leyendose 
	 * en este caso las X de menor a mayor y las Y de mayor a menor. Los casos posibles son:
	 * <UL>
	 * <LI><B>X > 0; Y < 0;</B> {true, false}</LI>
	 * <LI><B>X > 0; Y > 0;</B> {true, true}</LI>
	 * <LI><B>X < 0; Y > 0;</B> {false, true}</LI>
	 * <LI><B>X < 0; Y < 0;</B> {false, false}</LI>
	 * </UL>
	 *  
	 * @return
	 */
	private boolean[] getOrientation(){
		boolean[] orientation = {true, false};
		return orientation;
	}
	
	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.cts.ICoordTrans)
	 */
	public Image updateImage(int width, int height, ICoordTrans rp) {
		return null;
	}
	
	public RasterBuf getRaster(int width, int height, ICoordTrans rp) {
		int line;
		RasterBuf raster = null;
					
		Point2D begin = worldToRaster(new Point2D.Double(v.minX(), v.maxY()));
		Point2D end = worldToRaster(new Point2D.Double(v.maxX(), v.minY()));
		
		//Comprobaciones
		if(begin.getX() < 0)
			begin.setLocation(0, begin.getY());
		if(begin.getY() > buf.getHeight())
			begin.setLocation(begin.getX(), buf.getHeight());
		if(end.getY() < 0)
			end.setLocation(begin.getX(), 0);
		if(end.getX() > buf.getWidth())
			begin.setLocation(buf.getWidth(), begin.getY());
		
		int alpha = (getAlpha() & 0xff) << 24;
		
		try {
			//Esta funcin se usa para la renderizacin, por eso se crean 4 bandas a pion fijo
			raster = new RasterBuf(getDataType(), width, height, 4, true);
		
			setBand(RED_BAND,   rBandNr);
			setBand(GREEN_BAND, gBandNr);
			setBand(BLUE_BAND,  bBandNr);
			
			double w = Math.abs(end.getX() - begin.getX());
			double h = Math.abs(end.getY() - begin.getY());
			double stepX = w / ((double)width);
			double stepY = h / ((double)height);
			int endLine = (((int)end.getY() + 1) < buf.getHeight()) ? ((int)end.getY() + 1) : buf.getHeight();
			int endCol = (((int)end.getX() + 1) < buf.getWidth()) ? ((int)end.getX() + 1) : buf.getWidth();
			double initY = begin.getY() - (int)begin.getY();
			double initX = begin.getX() - (int)begin.getX(); 
			
			switch(getDataType()){
			case RasterBuf.TYPE_BYTE: 
				//writeRasterBufByte(begin, end, alpha, raster, width, height, w, h, stepX, stepY, endLine, endCol, initY, initX);
				break;
			case RasterBuf.TYPE_SHORT:;	
				writeRasterBufShort(begin, end, alpha, raster, width, height, w, h, stepX, stepY, endLine, endCol, initY, initX);
				break;
			case RasterBuf.TYPE_INT:
				writeRasterBufInt(begin, end, alpha, raster, width, height, w, h, stepX, stepY, endLine, endCol, initY, initX);
				break;
			case RasterBuf.TYPE_FLOAT:
				writeRasterBufFloat(begin, end, alpha, raster, width, height, w, h, stepX, stepY, endLine, endCol, initY, initX);
				break;
			case RasterBuf.TYPE_DOUBLE:
				writeRasterBufDouble(begin, end, alpha, raster, width, height, w, h, stepX, stepY, endLine, endCol, initY, initX);
				break;
			case RasterBuf.TYPE_UNDEFINED:break;
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return raster;
	}
	
	private void writeRasterBufShort(Point2D begin, Point2D end, int alpha, RasterBuf rb, int widthImg, int heightImg,
			double w, double h, double stepX, double stepY, int endLine, int endCol, double l, double initX){
		short[][][] pRGBArray = new short[4][(int)Math.ceil(h) + 1][(int)Math.ceil(w) + 1];

		int x = 0, y = 0;
		for (int line = (int)begin.getY(); line < endLine ; line ++) {
			x = 0;
			for (int col = (int)begin.getX(); col < endCol; col ++){
				pRGBArray[0][y][x] = buf.getElemShort(line, col, rBandNr);
				pRGBArray[1][y][x] = buf.getElemShort(line, col, gBandNr);
				pRGBArray[2][y][x] = buf.getElemShort(line, col, bBandNr);
				pRGBArray[3][y][x] = (short)alpha;
				x ++;
			}
			y ++;
		}
		for (int line = 0; line < heightImg; line ++) {
			double c = initX;
			for (int col = 0; col < widthImg; col ++){
				try{ 
					for(int band = 0; band < 4; band ++)
						rb.setElemShort(line, col, band, pRGBArray[band][(int)l][(int)c]); 
				}catch(ArrayIndexOutOfBoundsException e){}
				c += stepX;
			}
			l += stepY;
		}
		
	}
	
	private void writeRasterBufInt(Point2D begin, Point2D end, int alpha, RasterBuf rb, int widthImg, int heightImg,
			double w, double h, double stepX, double stepY, int endLine, int endCol, double l, double initX){
		int[][][] pRGBArray = new int[4][(int)Math.ceil(h) + 1][(int)Math.ceil(w) + 1];

		int x = 0, y = 0;
		for (int line = (int)begin.getY(); line < endLine ; line ++) {
			x = 0;
			for (int col = (int)begin.getX(); col < endCol; col ++){
				pRGBArray[0][y][x] = buf.getElemInt(line, col, rBandNr);
				pRGBArray[1][y][x] = buf.getElemInt(line, col, gBandNr);
				pRGBArray[2][y][x] = buf.getElemInt(line, col, bBandNr);
				pRGBArray[3][y][x] = (short)alpha;
				x ++;
			}
			y ++;
		}
		for (int line = 0; line < heightImg; line ++) {
			double c = initX;
			for (int col = 0; col < widthImg; col ++){
				try{ 
					for(int band = 0; band < 4; band ++)
						rb.setElemInt(line, col, band, pRGBArray[band][(int)l][(int)c]); 
				}catch(ArrayIndexOutOfBoundsException e){}
				c += stepX;
			}
			l += stepY;
		}
		
	}
	
	private void writeRasterBufFloat(Point2D begin, Point2D end, int alpha, RasterBuf rb, int widthImg, int heightImg,
			double w, double h, double stepX, double stepY, int endLine, int endCol, double l, double initX){
		float[][][] pRGBArray = new float[4][(int)Math.ceil(h) + 1][(int)Math.ceil(w) + 1];

		int x = 0, y = 0;
		for (int line = (int)begin.getY(); line < endLine ; line ++) {
			x = 0;
			for (int col = (int)begin.getX(); col < endCol; col ++){
				pRGBArray[0][y][x] = buf.getElemFloat(line, col, rBandNr);
				pRGBArray[1][y][x] = buf.getElemFloat(line, col, gBandNr);
				pRGBArray[2][y][x] = buf.getElemFloat(line, col, bBandNr);
				pRGBArray[3][y][x] = (float)alpha;
				x ++;
			}
			y ++;
		}
		for (int line = 0; line < heightImg; line ++) {
			double c = initX;
			for (int col = 0; col < widthImg; col ++){
				try{ 
					for(int band = 0; band < 4; band ++)
						rb.setElemFloat(line, col, band, pRGBArray[band][(int)l][(int)c]); 
				}catch(ArrayIndexOutOfBoundsException e){}
				c += stepX;
			}
			l += stepY;
		}
		
	}
	
	private void writeRasterBufDouble(Point2D begin, Point2D end, int alpha, RasterBuf rb, int widthImg, int heightImg,
			double w, double h, double stepX, double stepY, int endLine, int endCol, double l, double initX){
		double[][][] pRGBArray = new double[4][(int)Math.ceil(h) + 1][(int)Math.ceil(w) + 1];

		int x = 0, y = 0;
		for (int line = (int)begin.getY(); line < endLine ; line ++) {
			x = 0;
			for (int col = (int)begin.getX(); col < endCol; col ++){
				pRGBArray[0][y][x] = buf.getElemDouble(line, col, rBandNr);
				pRGBArray[1][y][x] = buf.getElemDouble(line, col, gBandNr);
				pRGBArray[2][y][x] = buf.getElemDouble(line, col, bBandNr);
				pRGBArray[3][y][x] = (float)alpha;
				x ++;
			}
			y ++;
		}
		for (int line = 0; line < heightImg; line ++) {
			double c = initX;
			for (int col = 0; col < widthImg; col ++){
				try{ 
					for(int band = 0; band < 4; band ++)
						rb.setElemDouble(line, col, band, pRGBArray[band][(int)l][(int)c]); 
				}catch(ArrayIndexOutOfBoundsException e){}
				c += stepX;
			}
			l += stepY;
		}
		
	}
	
	/**
	 * Asigna al objeto Image los valores con los dato de la imagen contenidos en el 
	 * vector de enteros.
	 * @param image	imagen con los datos actuales
	 * @param startX	inicio de la posicin en X dentro de la imagen
	 * @param startY	inicio de la posicin en X dentro de la imagen
	 * @param w	Ancho de la imagen
	 * @param h	Alto de la imagen
	 * @param rgbArray	vector que contiene la banda que se va a sustituir
	 * @param offset	desplazamiento
	 * @param scansize	tamao de imagen recorrida por cada p
	 */
	protected void setRGBLine(BufferedImage image, int startX, int startY, int w, int h, int[] rgbArray, 
			 int offset, int scansize) {
		image.setRGB(startX, startY, w, h, rgbArray, offset, scansize);
	}
	
	/**
	 * Asigna al objeto Image la mezcla entre los valores que ya tiene y los valores 
	 * con los dato de la imagen contenidos en el vector de enteros. De los valores RGB
	 * que ya contiene se mantienen las bandas que no coinciden con el valor de flags. La
	 * banda correspondiente a flags es sustituida por los datos del vector.
	 * @param image	imagen con los datos actuales
	 * @param startX	inicio de la posicin en X dentro de la imagen
	 * @param startY	inicio de la posicin en X dentro de la imagen
	 * @param w	Ancho de la imagen
	 * @param h	Alto de la imagen
	 * @param rgbArray	vector que contiene la banda que se va a sustituir
	 * @param offset	desplazamiento
	 * @param scansize	tamao de imagen recorrida por cada paso
	 * @param flags	banda que se va a sustituir (Ctes de GeoRasterFile)
	 */
	protected void setRGBLine(BufferedImage image, int startX, int startY, int w, int h, int[] rgbArray, 
			 int offset, int scansize, int flags) {
		int [] line = new int[rgbArray.length]; 
		image.getRGB(startX, startY, w, h, line, offset, scansize);
		if (flags == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x0000ffff) | (rgbArray[i] & 0xffff0000);
		else if (flags == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ff00ff) | (rgbArray[i] & 0xff00ff00);
		else if (flags == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ffff00) | (rgbArray[i] & 0xff0000ff);
		image.setRGB(startX, startY, w, h, line, offset, scansize);
	}
	
	/**
	 * Asigna al objeto Image la mezcla entre los valores que ya tiene y los valores 
	 * con los dato de la imagen contenidos en el vector de enteros. De los valores RGB
	 * que ya contiene se mantienen las bandas que no coinciden con el valor de flags. La
	 * banda correspondiente a flags es sustituida por los datos del vector.
	 * @param image	imagen con los datos actuales
	 * @param startX	inicio de la posicin en X dentro de la imagen
	 * @param startY	inicio de la posicin en X dentro de la imagen
	 * @param w	Ancho de la imagen
	 * @param h	Alto de la imagen
	 * @param rgbArray	vector que contiene la banda que se va a sustituir
	 * @param offset	desplazamiento
	 * @param scansize	tamao de imagen recorrida por cada paso
	 * @param origBand	Banda origen del GeoRasterFile
	 * @param destBandFlag	banda que se va a sustituir (Ctes de GeoRasterFile)
	 */
	protected void setRGBLine(BufferedImage image, int startX, int startY, int w, int h, int[] rgbArray, 
			 int offset, int scansize, int origBand, int destBandFlag) {
		int [] line = new int[rgbArray.length]; 
		image.getRGB(startX, startY, w, h, line, offset, scansize);
		if (origBand == 0 && destBandFlag == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x0000ffff) | (rgbArray[i] & 0xffff0000);
		else if (origBand == 1 && destBandFlag == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ff00ff) | (rgbArray[i] & 0xff00ff00);
		else if (origBand == 2 && destBandFlag == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ffff00) | (rgbArray[i] & 0xff0000ff);
		
		else if (origBand == 0 && destBandFlag == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffff00ff) | ((rgbArray[i] & 0x00ff0000) >> 8) ;
		else if (origBand == 0 && destBandFlag == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffffff00) | ((rgbArray[i] & 0x00ff0000) >> 16);
		else if (origBand == 1 && destBandFlag == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xff00ffff) | ((rgbArray[i] & 0x0000ff00) << 8);
		
		else if (origBand == 1 && destBandFlag == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffffff00) | ((rgbArray[i] & 0x0000ff00) >> 8);
		else if (origBand == 2 && destBandFlag == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xff00ffff) | ((rgbArray[i] & 0x000000ff) << 16);
		else if (origBand == 2 && destBandFlag == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffff00ff) | ((rgbArray[i] & 0x000000ff) << 8);
		image.setRGB(startX, startY, w, h, line, offset, scansize);
	}
		
	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.cts.ICoordTrans, java.awt.Image, int, int)
	 */
	public Image updateImage(int width, int height, ICoordTrans rp, Image img, int origBand, int destBandFlag)throws SupersamplingNotSupportedException{
		int pRGBArray[] = null;

		if(width<=0)width=1;
		if(height<=0)height=1;
		Point2D begin = worldToRaster(new Point2D.Double(v.minX(), v.maxY()));
		Point2D end = worldToRaster(new Point2D.Double(v.maxX(), v.minY()));
		
		//Comprobaciones
		if(begin.getX() < 0)
			begin.setLocation(0, begin.getY());
		if(begin.getY() > buf.getHeight())
			begin.setLocation(begin.getX(), buf.getHeight());
		if(end.getY() < 0)
			end.setLocation(begin.getX(), 0);
		if(end.getX() > buf.getWidth())
			begin.setLocation(buf.getWidth(), begin.getY());
		
		int alpha = (getAlpha() & 0xff) << 24;
				
		//TODO: Falta orientacin
		try {
			setBand(RED_BAND,   rBandNr);
			setBand(GREEN_BAND, gBandNr);
			setBand(BLUE_BAND,  bBandNr);
			if(img!=null){
				writeImage(begin, end, alpha, origBand, destBandFlag, img, width, height, true);
				return img;
			}else{
				Image image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
				writeImage(begin, end, alpha, origBand, destBandFlag, image, width, height, false);
				return image;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return img;
	}
			
	private void writeImage(Point2D begin, Point2D end, int alpha, int origBand, int destBandFlag, 
			Image img, int widthImg, int heightImg, boolean existsImg){
		double w = Math.abs(end.getX() - begin.getX());
		double h = Math.abs(end.getY() - begin.getY());
		double stepX = w / ((double)widthImg);
		double stepY = h / ((double)heightImg);
				
		int x = 0, y = 0;
		int[][] pRGBArray = new int[(int)Math.ceil(h) + 1][(int)Math.ceil(w) + 1];
		int endLine = (((int)end.getY() + 1) < buf.getHeight()) ? ((int)end.getY() + 1) : buf.getHeight();
		int endCol = (((int)end.getX() + 1) < buf.getWidth()) ? ((int)end.getX() + 1) : buf.getWidth();
		if(buf.getDataType() == RasterBuf.TYPE_BYTE){
			for (int line = (int)begin.getY(); line < endLine ; line ++) {
				x = 0;
				for (int col = (int)begin.getX(); col < endCol; col ++){
					int value = (alpha + 
								((buf.getElemByte(line, col, rBandNr)& 0xff) << 16) +
								((buf.getElemByte(line, col, gBandNr)& 0xff) << 8) +
								(buf.getElemByte(line, col, bBandNr)& 0xff));
					pRGBArray[y][x] = value;
					x++;
				}
				y++;
			}
			int[] rgbImgLine = new int[widthImg];
			double l = begin.getY() - (int)begin.getY();
			double initX = begin.getX() - (int)begin.getX(); 
			for (int line = 0; line < heightImg; line ++) {
				double c = initX;
				for (int col = 0; col < widthImg; col ++){
					try{
						rgbImgLine[col] = pRGBArray[(int)l][(int)c];
					}catch(ArrayIndexOutOfBoundsException e){}
					c += stepX;
				}
				if(existsImg)
					setRGBLine((BufferedImage) img, 0, line, widthImg, 1, rgbImgLine, 0, widthImg, origBand, destBandFlag);
				else
					setRGBLine((BufferedImage) img, 0, line, widthImg, 1, rgbImgLine, 0, widthImg);
				l += stepY;
			}
			
		}
	}
	
	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#getData(int, int, int)
	 */
	public Object getData(int x, int y, int band) {
		if(buf.getDataType() == RasterBuf.TYPE_BYTE){
			return new Byte(buf.getElemByte(y, x, band));
		}else if(buf.getDataType() == RasterBuf.TYPE_SHORT){
			return new Short(buf.getElemShort(y, x, band));
		}else if(buf.getDataType() == RasterBuf.TYPE_INT){
			return new Integer(buf.getElemInt(y, x, band));
		}else if(buf.getDataType() == RasterBuf.TYPE_FLOAT){
			return new Float(buf.getElemFloat(y, x, band));
		}else if(buf.getDataType() == RasterBuf.TYPE_DOUBLE){
			return new Double(buf.getElemDouble(y, x, band));
		}
		return null;
	}
	
	/**
	 * Devuelve los datos de una ventana solicitada
	 * @param ulX	coordenada X superior izda.
	 * @param ulY	coordenada Y superior derecha.
	 * @param sizeX	tamao en X de la ventana.
	 * @param sizeY tamao en Y de la ventana.
	 * @param band	Banda solicitada.
	 */
	public byte[] getWindow(int ulX, int ulY, int sizeX, int sizeY, int band){
		
		return null;
	}
	
	public RasterBuf getWindowRaster(double x, double y, double w, double h, BandList bandList, RasterBuf rasterBuf) {		
		Extent selectedExtent = new Extent(x, y, x + w, y - h);
		setView(selectedExtent);
		
		int width = 0;
		int height = 0;
		if(getTransform() != null){
			width = (int)Math.abs(selectedExtent.width() / getTransform()[1]);//(int)(selectedExtent.width() * file.width) / extent.width();
			height = (int)Math.abs(selectedExtent.height() / getTransform()[5]);
		}else{
			width = (int)Math.abs(selectedExtent.width());
			height = (int)Math.abs(selectedExtent.height());
		}
				
		/*try {
			file.readWindow(rasterBuf, bandList, x, y, width, height);
		} catch (Exception e) {
			e.printStackTrace();
		}*/
		
		return rasterBuf;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.fmap.driver.GeoRasterFile#getWindowRaster(double, double, double, double, int, int, org.gvsig.fmap.driver.BandList, org.gvsig.fmap.driver.IBuffer)
	 */
	public RasterBuf getWindowRaster(double minX, double minY, double maxX, double maxY, int bufWidth, int bufHeight, BandList bandList, RasterBuf rasterBuf) {		
		Extent selectedExtent = new Extent(minX, minY, maxX, maxY);
		setView(selectedExtent);
		
		double width = 0;
		double height = 0;
		if(getTransform() != null){
			width = (double)(Math.abs(selectedExtent.width() / getTransform()[1]));//(int)(selectedExtent.width() * file.width) / extent.width();
			height = (double)(Math.abs(selectedExtent.height() / getTransform()[5]));
		}else{
			width = (double)Math.abs(selectedExtent.width());
			height = (double)Math.abs(selectedExtent.height());
		}
				
		/*try {
			file.readWindow(rasterBuf, bandList, minX, maxY, maxX, minY, width, height, bufWidth, bufHeight);
		} catch (Exception e) {
			e.printStackTrace();
		}*/
		
		return rasterBuf;
	}
	
	public RasterBuf getWindowRaster(int x, int y, int w, int h, BandList bandList, RasterBuf rasterBuf) {
		for(int iBand = 0; iBand < buf.getBandCount(); iBand ++){
			int[] drawableBands = bandList.getBufferBandToDraw(null, iBand);
			if(drawableBands == null || (drawableBands.length == 1 && drawableBands[0] == -1))
				continue;	
			if(buf.getDataType() == RasterBuf.TYPE_BYTE){
				for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){
					for(int line = y; line < (y + h); line ++)
						for(int col = x; col < (x + w); col ++)
							rasterBuf.setElemByte((line - y), (col - x), drawableBands[drawBands], buf.getElemByte(line, col, drawableBands[drawBands]));
				}
			}else if(buf.getDataType() == RasterBuf.TYPE_SHORT){
				for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){
					for(int line = y; line < (y + h); line ++)
						for(int col = x; col < (x + w); col ++)
							rasterBuf.setElemShort((line - y), (col - x), drawableBands[drawBands], buf.getElemShort(line, col, drawableBands[drawBands]));
				}
			}else if(buf.getDataType() == RasterBuf.TYPE_INT){
				for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){
					for(int line = y; line < (y + h); line ++)
						for(int col = x; col < (x + w); col ++)
							rasterBuf.setElemInt((line - y), (col - x), drawableBands[drawBands], buf.getElemInt(line, col, drawableBands[drawBands]));
				}
			}else if(buf.getDataType() == RasterBuf.TYPE_FLOAT){
				for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){
					for(int line = y; line < (y + h); line ++)
						for(int col = x; col < (x + w); col ++)
							rasterBuf.setElemFloat((line - y), (col - x), drawableBands[drawBands], buf.getElemFloat(line, col, drawableBands[drawBands]));
				}
			}else if(buf.getDataType() == RasterBuf.TYPE_DOUBLE){
				for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){
					for(int line = y; line < (y + h); line ++)
						for(int col = x; col < (x + w); col ++)
							rasterBuf.setElemDouble((line - y), (col - x), drawableBands[drawBands], buf.getElemDouble(line, col, drawableBands[drawBands]));
				}
			}
		}
		return rasterBuf;
	}
	
	public RasterBuf getWindowRasterWithNoData(double x, double y, double w, double h, BandList bandList, RasterBuf rasterBuf) {
		return null;
	}
	
	/**
	 * Obtiene la zona (Norte / Sur)
	 * @return true si la zona es norte y false si es sur
	 */
	
	public boolean getZone(){
		return false;
	}
	
	/**
	 *Devuelve el nmero de zona UTM
	 *@return Nmero de zona 
	 */
	
	public int getUTM(){
		return 0;	
	}
	
	/**
	 * Obtiene el sistema de coordenadas geograficas
	 * @return Sistema de coordenadas geogrficas
	 */
	public String getGeogCS(){
		return new String("");	
	}
	
	/**
	 * Devuelve el tamao de bloque
	 * @return Tamao de bloque
	 */
	public int getBlockSize(){
		return 0;
	}
	
	/**
	 * Obtiene el objeto que contiene los metadatos
	 */
	public Metadata getMetadata() {
		return null;
	}
	
	/**
	 * Obtiene el flag que dice si la imagen est o no georreferenciada
	 * @return true si est georreferenciada y false si no lo est.
	 */
	public boolean isGeoreferenced() {
		return (this.extent != null);
	}
    
	/**
	 * Informa de si el driver ha supersampleado en el ltimo dibujado. Es el driver el que colocar
	 * el valor de esta variable cada vez que dibuja. 
	 * @return true si se ha supersampleado y false si no se ha hecho.
	 */
	public boolean isSupersampling() {
		return false;
	}
	
	/**
	 * Obtiene los parmetros de la transformacin afn que corresponde con los elementos de
	 * un fichero tfw.
	 * <UL> 
	 * <LI>[1]tamao de pixel en X</LI>
	 * <LI>[2]rotacin en X</LI>
	 * <LI>[4]rotacin en Y</LI>
	 * <LI>[5]tamao de pixel en Y</LI>
	 * <LI>[0]origen en X</LI>
	 * <LI>[3]origen en Y</LI>
	 * </UL>
	 * Este mtodo debe ser reimplementado por el driver si tiene esta informacin. En principio
	 * Gdal es capaz de proporcionarla de esta forma.
	 * 
	 * En caso de que exista fichero .rmf asociado al raster pasaremos de la informacin de georreferenciacin
	 * del .tfw y devolveremos la que est asociada al rmf
	 * @return vector de double con los elementos de la transformacin afn.
	 */
	public double[] getTransform(){
			return null;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.fmap.driver.GeoRasterFile#rasterToWorld(java.awt.geom.Point2D)
	 */
	public Point2D rasterToWorld(Point2D pt) {
		double x = extent.minX() + ((pt.getX() * (extent.maxX() - extent.minX())) / ((double) buf.getWidth()));
		double y = extent.maxY() - ((pt.getY() * (extent.maxY() - extent.minY())) / ((double) buf.getHeight()));
		Point2D ptRes = new Point2D.Double(x, y);
		return ptRes;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.fmap.driver.GeoRasterFile#worldToRaster(java.awt.geom.Point2D)
	 */
	public Point2D worldToRaster(Point2D pt) {
		double x = (((double) buf.getWidth()) / (extent.maxX() - extent.minX())) * (pt.getX() - extent.minX());
		double y = (((double) buf.getHeight()) / (extent.maxY() - extent.minY())) * (extent.maxY() - pt.getY());
		Point2D ptRes = new Point2D.Double(x, y);
		return ptRes;
	}
	
	/**
	 * @return Returns the dataType.
	 */
	public int getDataType() {
		return buf.getDataType();
	}
}



