/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.io.data;

import java.util.ArrayList;

import org.cresques.io.exceptions.BandFoundInListException;

/**
 * Esta clase gestiona una lista de bandas que puede pertenecer
 * a un fichero (la tiene un geoRasterFile) o puede ser una lista 
 * de bandas de multiples ficheros (la tiene un multifiledataset) 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class BandList {
	//Band array
	private ArrayList bands = new ArrayList();
			
	/**
	 * Encuentra una banda en la lista.
	 * @param file Fichero al que pertenece la banda buscada.
	 * @param pos Posicin que ocupa en el fichero.
	 * @return true si se ha hallado la banda y false si no se 
	 * ha encontrado
	 */
	public boolean findBand(Band band){
		for(int i = 0; i < bands.size(); i++){
			Band b = (Band)bands.get(i); 
			if(	b.getFileName().equals(band.getFileName()) &&
				b.getPosition() == band.getPosition())
				return true;
		}
		return false;
	}
		
	/**
	 * Obtiene la lista de bandas a pintar sobre el buffer de 
	 * salida en forma de array. Cada elemento del array es una banda
	 * del RasterBuf de salida que ha de dibujarse y el valor que contiene
	 * ese elemento es la banda de la imagen que va pintada en esa banda 
	 * del RasterBuf.
	 * <table border=1>
	 * <tr><td>Elem</td><td>Valor</td></tr>
	 * <tr><td>0</td><td>1</td></tr>
	 * <tr><td>1</td><td>0</td></tr>
	 * <tr><td>2</td><td>-1</td></tr>
	 * <tr><td>3</td><td>0</td></tr>
	 * <tr><td>4</td><td>2</td></tr>
	 * </table>
	 * El RasterBuf tendra en la banda 0 dibujada la banda 1 de este GeoRasterFile, 
	 * en la banda 1 la 0 del dataset, en la banda 2 no habr ninguna de este dataset,
	 * en la 3 la 0 y en la 4 la 2 de este GeoRasterFile.
	 * @return lista de bandas a dibujar o un array de un elemento con valor -1.
	 */
	public int[] bandsToDrawList(int nBandsDataImage){
		
		int[] bandsToRead = new int[nBandsDataImage];
		for(int dataImageBand = 0;dataImageBand < nBandsDataImage;dataImageBand++){
			bandsToRead[dataImageBand] = -1;
			for(int band = 0;band < bands.size();band++){
				if(	((Band)bands.get(band)) != null &&
					((Band)bands.get(band)).getDataImageBandToDraw() != null){
					for(int dest=0;dest<((Band)bands.get(band)).getDataImageBandToDraw().length;dest++){
						if(dataImageBand == ((Band)bands.get(band)).getDataImageBandToDraw()[dest])
							bandsToRead[dataImageBand] = band;
					}
				}
			}
		}
	
		return bandsToRead;
	}
	
	/**
	 * Checkea si alguna banda de la lista se esta dibujando en el buffer de salida.
	 * @return	true si alguna banda se est dibujando y false si no
	 */
	public boolean isDrawingAnyBand(){
		for(int band = 0;band < bands.size();band++){
			if(((Band)bands.get(band)).isDrawing())
				return true;
		}
		return false;
	}
	
	//******************************
	//Setters and Getters
	//******************************
	
	/**
	 * Aade una banda a la lista.
	 * @param b banda a aadir.
	 */
	public void addBand(Band b, int pos)throws BandFoundInListException{
		bands.add(b);
	}
	
	/**
	 * Aade la lista de bandas pasada por parmetro a la lista
	 * actual. Si alguna banda ya existe no la aade y continua.
	 * @param bl Lista de bandas
	 */
	public void addBandList(BandList bl){
		for(int i = 0; i < bl.getBandCount(); i++){
			if(!findBand(bl.getBand(i)))
				bands.add(bl.getBand(i));
		}
	}
	
	/**
	 * ELimina todas las bandas que tienen un nombre determinado.
	 * @param name Nombre de las bandas a eliminar
	 */
	public void removeBands(String name){
		for(int i = 0; i < getBandCount(); i++){
			Band band = getBand(i);
			if(band.getFileName().equals(name))
				bands.remove(i);
		}
	}
	
	/**
	 * Resetea la asignacin de dibujado de las bandas de la imagen
	 * sobre el DataImage cuando se hace un update para esta banda.
	 */
	public void clearDrawableBand(){
		for(int i = 0; i < getBandCount(); i++)
			((Band)bands.get(i)).clearDrawableBands();
	}
	
	/**
	 * Para este GeoRasterFile asigna que bandas se pintaran
	 * sobre el RasterBuf cuando se haga un update. Especificamos a 
	 * travs de los parmetros para que posicin del RasterBuf ir 
	 * dibujada con que banda del fichero de imagen.
	 * @param posDataImage	Posicin del RasterBuf que queremos pintar.
	 * @param imageBand	Banda de la imagen que se pintar
	 */
	public void addDrawableBand(int posRasterBuf, int imageBand){
		try{
			((Band)bands.get(imageBand)).setPositionToDrawInBuffer(posRasterBuf);
		}catch(IndexOutOfBoundsException exc){
			//No hacemos nada simplemente no inserta la banda.
		}
	}
	
	/**
	 * Obtiene el nmero de bandas de un RasterBuf sobre las que se pintara
	 * alguna banda de este fichero cuando se llama a un updateBuffer. 
	 * @return Nmero de bandas. Cero en caso de no tener ninguna asignada. 
	 */
	public int getDrawableBandsCount(){
		int nbands = 0;
		for(int i = 0;i<bands.size();i++){
			Band b = (Band)bands.get(i);
			if(b.getDataImageBandToDraw() != null)
				nbands += b.getDataImageBandToDraw().length;
		}
		return nbands;
	}
	
	/**
	 * Obtiene la banda de la posicin i.
	 * @param i Posicin de la banda a obtener.
	 * @return Banda.
	 */
	public Band getBand(int i){
		return (Band)bands.get(i);
	}
	
	/**
	 * Obtiene el nmero de bandas.
	 * @return entero con el nmero de bandas.
	 */
	public int getBandCount(){
		return bands.size();
	}
	
	/**
	 * A partir de un nombre de fichero y un nmero de banda obtiene la banda o bandas del buffer de salida 
	 * donde se pinta.
	 * @param fileName Nombre de fichero
	 * @param band Nmero de banda de la imagen
	 * @return banda o bandas del buffer de salida donde se dibuja.
	 */
	public int[] getBufferBandToDraw(String fileName, int band){
		if(fileName != null){
			for(int i = 0; i < bands.size(); i++){
				if(((Band)bands.get(i)).getFileName().equals(fileName))
					if(((Band)bands.get(i)).getPosition() == band)
						return ((Band)bands.get(i)).getDataImageBandToDraw();
			}
		}else{
			for(int i = 0; i < bands.size(); i++){
				if(((Band)bands.get(i)).getPosition() == band)
					return ((Band)bands.get(i)).getDataImageBandToDraw();
			}
		}
		int[] r = {-1};
		return r;
	}
	/**
	 * Obtiene en un array de String la lista de nombres de ficheros
	 * @return lista de nombres de los ficheros del GeoRasterMultiFile
	 */
	public String[] getBandStringList(){
		String[] list = new String[bands.size()];
		for(int i = 0; i < bands.size(); i++)
			list[i] = ((Band)bands.get(i)).getFileName();
		return list;
	}
	
	/**
	 * Obtiene en un array de enteros con la lista de la posicin de bandas
	 * @return lista de la posicin de bandas
	 */
	public int[] getBandPositionList(){
		int[] list = new int[bands.size()];
		for(int i = 0; i < bands.size(); i++)
			list[i] = ((Band)bands.get(i)).getPosition();
		return list;
	}
	
	/**
	 * Obtiene el tipo de dato de las bandas. Esta llamada supone que todos los tipos de dato
	 * de las bandas son igual por lo que devolver el primero. 
	 * @return Entero que representa el tipo de datos de las bandas
	 */
	public int getBandsDataType(){
		if(bands.size() <= 0)
			return RasterBuf.TYPE_UNDEFINED;
		return ((Band)bands.get(0)).getDataType();
	}
	 
}
