/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.io.data;

import org.cresques.io.Rasterizer;

/**
 * Salva a raster un objeto Grid.
 * Sirve datos solicitados por los drivers que salvan a raster. 
 * Hereda de Rasterizer y reescribe el mtodo readData que es el que 
 * ser llamado desde el driver cada vez que vacie el buffer y 
 * necesite ms datos.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class RasterizeGrid extends Rasterizer{
	
	public static final int			BLOCKSIZE = 512;
	
	/**
	 * Nmero de bloques totales a leer del grid
	 */
	private int						nBlocks = 0;
	/**
	 * Tamao en pixeles del ltimo bloque
	 */
	private int						sizeLastBlock = 0;
	/**
	 * Posicin y del grid donde se ha quedado la lectura para que el la siguiente
	 * peticin se sirva desde donde se qued la ltima vez.
	 */
	private int 					line = 0;
	/**
	 * Buffer con los datos del grid
	 */
	private RasterBuf 				rasterBuf = null;
	/**
	 * Bandas a salvar. Si vale null se salvarn todas las del rasterBuf sino se har
	 * una seleccin. Cada elemento del array es una banda del rasterBuf y el valor del contenido
	 * es la posicin en la que se salva.
	 */
	private int[]					bands = null;
	/**
	 * Cuando se desea salvar un raster con una sola banda del rasterBuf bands valdr null y se 
	 * usar band para saber que banda del rasterBuf se va a salvar. Esto indica usando el constructor
	 * adecuado.
	 */
	private int						band = 0;
	/**
	 * Constructor
	 * @param grid Grid desde donde se leen los datos
	 * @param bands Bandas a salvar. Si vale null se salvarn todas las del rasterBuf sino se har
	 * una seleccin. Cada elemento del array es una banda del rasterBuf y el valor del contenido
	 * es la posicin en la que se salva.
	 */
	public RasterizeGrid(RasterBuf rasterBuf, int[] bands){
		super(BLOCKSIZE);	
		this.bands = bands;
		
		//Cuando bands es null es que entran todas las bandas
		if(bands == null){
			bands = new int[rasterBuf.getBandCount()];
			for (int iBand = 0; iBand < rasterBuf.getBandCount(); iBand++)
				bands[iBand] = iBand;
		}
				
		init(rasterBuf);
	}
	
	/**
	 * Constructor
	 * @param rasterBuf RasterBuf desde donde se leen los datos
	 * @param bands Bandas a salvar. Si vale null se salvarn todas las del rasterBuf sino se har
	 * una seleccin. Cada elemento del array es una banda del rasterBuf y el valor del contenido
	 * es la posicin en la que se salva.
	 */
	public RasterizeGrid(RasterBuf rasterBuf, int band){
		super(BLOCKSIZE);
		this.band = band;
		init(rasterBuf);
	}
	
	/**
	 * Inicializa el nmero de bloques y el tamao del ltimo. Asigna el grid
	 * y el rasterbuf desde el que se obtienen los datos.
	 * @param grid
	 */
	public void init(RasterBuf rasterBuf){
		this.rasterBuf = rasterBuf;;
		nBlocks = (int)(rasterBuf.getHeight() / BLOCKSIZE);
		sizeLastBlock = (rasterBuf.getHeight() - (nBlocks * BLOCKSIZE));
		nBlocks ++;
	}
	
	/* (non-Javadoc)
	 * @see org.cresques.io.IDataWriter#readData(int, int, int)
	 */
	public int[] readARGBData(int sX, int sY, int nBand){
		return null;
	}
	
	/**
	 * Lee los datos tipo byte del grid de ancho sizeX y alto sizeY desde la posicin
	 * indicada en la variable de instancia "line". El array bands dice que bandas del
	 * rasterBuf va a ser salvadas, en caso de que bands sea null significa que solo vamos
	 * a salvar una banda del rasterBuf con lo que usaremos la variable band que indica que
	 * banda hay que salvar.
	 */
	public byte[][] readByteData(int sizeX, int sizeY) {
		byte[][] buffer = null;
		int cont = 0;
		
		if(bands != null){
			buffer = new byte[bands.length][sizeX * sizeY];
			for (int iBand = 0; iBand < bands.length; iBand++){
				cont = 0;
				for(int iRow = line; iRow < (line + sizeY); iRow ++){
					for(int iCol = 0; iCol < sizeX; iCol ++){
						buffer[iBand][cont] = rasterBuf.getElemByte(iRow, iCol, iBand);
						cont ++;
					}
				}
			}
		}else{
			buffer = new byte[1][sizeX * sizeY];
			for(int iRow = line; iRow < (line + sizeY); iRow ++){
				for(int iCol = 0; iCol < sizeX; iCol ++){
					buffer[0][cont] = rasterBuf.getElemByte(iRow, iCol, band);
					cont ++;
				}
			}
		}
				
		line += sizeY;
		return buffer;
	}

	public short[][] readShortData(int sizeX, int sizeY) {
		short[][] buffer = null;
		int cont = 0;
		
		if(bands != null){
			buffer = new short[bands.length][sizeX * sizeY];
			for (int iBand = 0; iBand < bands.length; iBand++){
				cont = 0;
				for(int iRow = line; iRow < (line + sizeY); iRow ++){
					for(int iCol = 0; iCol < sizeX; iCol ++){
						buffer[iBand][cont] = rasterBuf.getElemShort(iRow, iCol, iBand);
						cont ++;
					}
				}
			}
		}else{
			buffer = new short[1][sizeX * sizeY];
			for(int iRow = line; iRow < (line + sizeY); iRow ++){
				for(int iCol = 0; iCol < sizeX; iCol ++){
					buffer[0][cont] = rasterBuf.getElemShort(iRow, iCol, band);
					cont ++;
				}
			}
		}		
				
		line += sizeY;
		return buffer;
	}

	public int[][] readIntData(int sizeX, int sizeY) {
		int[][] buffer = null;
		int cont = 0;
		
		if(bands != null){
			buffer = new int[bands.length][sizeX * sizeY];
			for (int iBand = 0; iBand < bands.length; iBand++){
				cont = 0;
				for(int iRow = line; iRow < (line + sizeY); iRow ++){
					for(int iCol = 0; iCol < sizeX; iCol ++){
						buffer[iBand][cont] = rasterBuf.getElemInt(iRow, iCol, iBand);
						cont ++;
					}
				}
			}
		}else{
			buffer = new int[1][sizeX * sizeY];
			for(int iRow = line; iRow < (line + sizeY); iRow ++){
				for(int iCol = 0; iCol < sizeX; iCol ++){
					buffer[0][cont] = rasterBuf.getElemInt(iRow, iCol, band);
					cont ++;
				}
			}
		}			
		
		line += sizeY;
		return buffer;
	}
	
	public float[][] readFloatData(int sizeX, int sizeY) {
		float[][] buffer = null;
		int cont = 0;
		
		if(bands != null){
			buffer = new float[bands.length][sizeX * sizeY];
			for (int iBand = 0; iBand < bands.length; iBand++){
				cont = 0;
				for(int iRow = line; iRow < (line + sizeY); iRow ++){
					for(int iCol = 0; iCol < sizeX; iCol ++){
						buffer[iBand][cont] = rasterBuf.getElemFloat(iRow, iCol, iBand);
						cont ++;
					}
				}
			}
		}else{
			buffer = new float[1][sizeX * sizeY];
			for(int iRow = line; iRow < (line + sizeY); iRow ++){
				for(int iCol = 0; iCol < sizeX; iCol ++){
					buffer[0][cont] = rasterBuf.getElemFloat(iRow, iCol, band);
					cont ++;
				}
			}
		}				
		
		line += sizeY;
		return buffer;
	}

	public double[][] readDoubleData(int sizeX, int sizeY) {
		double[][] buffer = null;
		int cont = 0;
		
		if(bands != null){
			buffer = new double[bands.length][sizeX * sizeY];
			for (int iBand = 0; iBand < bands.length; iBand++){
				cont = 0;
				for(int iRow = line; iRow < (line + sizeY); iRow ++){
					for(int iCol = 0; iCol < sizeX; iCol ++){
						buffer[iBand][cont] = rasterBuf.getElemDouble(iRow, iCol, iBand);
						cont ++;
					}
				}
			}
		}else{
			buffer = new double[1][sizeX * sizeY];
			for(int iRow = line; iRow < (line + sizeY); iRow ++){
				for(int iCol = 0; iCol < sizeX; iCol ++){
					buffer[0][cont] = rasterBuf.getElemDouble(iRow, iCol, band);
					cont ++;
				}
			}
		}				
		
		line += sizeY;
		return buffer;
	}

}