/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.datastruct;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Writer;

import org.cresques.io.data.Grid;
import org.cresques.io.data.RasterBuf;
import org.cresques.px.Extent;
import org.cresques.px.PxRaster;
import org.cresques.util.Utilities;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;


/**
 * Clase para la gestin de histogramas de un raster. Es la encargada del calculo de un histograma
 * total o parcial de un raster a partir de los datos de disco. Este calculo se hace leyendo los
 * datos por bloques para no cargarlo completamente en memoria. Adems tambin es la encargada de gestionar
 * salvar este histograma a .rmf. En caso de que solicite un histograma del raster completo este ir
 * a buscarlo al fichero rmf asociado antes de calcularlo por si ya existise. Al realizar el calculo
 * del histograma de la imagen completa este ser salvado en el fichero .rmf asociado al raster.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class Histogram extends Thread{
	
	/**
	 * Tamao del bloque de calculo de histograma. La lectura del raster se divide en partes
	 * de BLOCK de altura para no cargar todo el raster en memoria de una vez.
	 */
	private static final int			BLOCK = 512;
	private Grid 						grid = null;

	/**
	 * Flag que dice si el histograma a calcualar es el seleccionado en el area de interes
	 * o el histograma completo.
	 */ 
	private boolean 					calcFullHistogram = true;
	/**
	 * Histograma de la imagen completa
	 */
	private int[][]						histogram = null;
	/**
	 * Histograma acumulado de la imagen completa
	 */
	private int[][]						accumulatedHistogram = null;
	/**
	 * Variables que definen una rea de interes
	 */
	private int 						xAOI, yAOI, widthAOI, heightAOI;
	/**
	 * Porcentaje que lleva construido el histograma
	 */
	private int							percent = 0;
	/**
	 * Variable que estar a true si el histograma esta siendo leido desde fichero .rmf y false
	 * si ha sido calculado. En este ltimo caso se salvar a disco una vez termine de ser calculado
	 */
	private boolean[]					isReadingFromRMF = null;
	/**
	 * Renderizador para el calculo de histogramas a partir 
	 * de la visualizacin.
	 */
	private PxRaster					pxRaster = null;
	/**
	 * Histograma de la vista
	 */
	private int[][]						viewHistogram = null;
	/**
	 * Bandas que solicita el cliente. Cada elemento del array es una banda, si est a true se devuelve 
	 * esa banda con histograma calculado y si est a false se devolver esa banda con el histograma a 0.
	 * Este array solo es tendo en cuenta para el histograma con fuente de datos reales y no para el de
	 * la visualizacin.
	 */
	private boolean[]					showBands = {true, true, true};
	/**
	 * Writer para modificar o crear los archivos .rmf.
	 */
	private Writer						writer = null;
	/**
	 * Herramientas para la persistencia del histograma;
	 */
	private HistogramPersistence				tools = null;
	/**
	 * Constructor 
	 * @param grid
	 */
	public Histogram(Grid grid){
		addGrid(grid);
	}
	
	/**
	 * Sustituye el grid del histograma 
	 * @param grid
	 */
	public void addGrid(Grid grid){
		this.grid = grid;
		isReadingFromRMF = new boolean[grid.getBandCount()];
	}

	/**
	 * Obtiene el grid asociado al histograma
	 * @return Grid
	 */
	public Grid getGrid() {
		return grid;
	}
	
	/**
	 * Obtiene el renderizador asociado al histograma
	 * @return PxRaster
	 */
	public PxRaster getRenderizer() {
		return pxRaster;
	}
	
	/**
	 * Asigna el renderizador para el calculo de histogramas a partir 
	 * de la visualizacin.
	 * @param pxR Renderizador.
	 */
	public void addRenderizer(PxRaster pxR){
		this.pxRaster = pxR;
	}
	
	/**
	 * Asigna un area de interes para el calculo de histograma
	 * @param x Posicin X del area de interes
	 * @param y Posicin Y del area de interes
	 * @param w Ancho del rea de interes
	 * @param h Alto del rea de interes
	 */
	public void setAreaOfInterest(int x, int y, int w, int h){
		this.xAOI = x;
		this.yAOI = y;
		this.widthAOI = w;
		this.heightAOI = h;
	}
	
	/**
	 * Obtiene el histograma de la imagen completa en caso de que este ya haya sido
	 * calculado. Si histogram es null habr que llamar a la funcin calcFullHistogram para
	 * calcularlo.
	 * @return histograma de la imagen completa
	 */
	public int[][] getFullHistogram(){
		return histogram;
	}
	
	/**
	 * Obtiene el histograma acumulado de la imagen completa en caso de que este ya haya sido
	 * calculado. Si histogram es null habr que llamar a la funcin calcFullAccumulatedHistogram para
	 * calcularlo.
	 * @return histograma acumulado de la imagen completa
	 */
	public int[][] getFullAccumulatedHistogram(){
		histogram = null;
		histogram = readFullHistogramFromRMF();
		
		return accumulatedHistogram;
	}
	
	/**
	 * Obtiene el histograma desde el fichero RMF si tiene
	 * @return 
	 */
	private int[][] readFullHistogramFromRMF(){
		String fName = null;
		String values = null;
		String[] vect = null;
		File fl = null;
		KXmlParser parser = new KXmlParser();
		int [][] hist = null;
		int index = 0;
		boolean end = false;
		boolean exists = false;
		
		hist = new int [grid.getGeoRasterMultiFile().getBandCount()][256];
		try{
			for(int i = 0 ; i < grid.getGeoRasterMultiFile().getFileCount() ; i++){
				fName = grid.getGeoRasterMultiFile().getFile(i).getName();
				fl = new File(fName.substring(0, fName.lastIndexOf(".") + 1) + "rmf");
				exists = false;
				if (!fl.exists())
					return null;
				else{
					parser.setInput(new FileInputStream(fl.getPath()), null);
					int tag = parser.next();
					parser.require(parser.START_TAG, null, "RasterMetaFile");
					tag = parser.next();
					end = false;
					while(!end){
						switch(tag){
						case KXmlParser.TEXT:
							tag = parser.next();
							break;
						case KXmlParser.START_TAG:
							if (parser.getName().equals("Band")){
								parser.require(KXmlParser.START_TAG, null, "Band");
								parser.nextTag();
								parser.require(KXmlParser.START_TAG, null, "Values");
								values = parser.nextText();
								parser.require(KXmlParser.END_TAG, null, "Values");
								tag = parser.nextTag();
								parser.require(KXmlParser.END_TAG, null, "Band");
								
								exists = true;
								vect = new String[256];
								vect = values.split(" ");
								
								for(int j = 0 ; j < 256 ; j++){
									if (showBands[index])
										hist[index][j] = Integer.valueOf(vect[j]).intValue();
									else
										hist[index][j] = 0;
								}
									
								index++;
							}
							tag = parser.next();
							break;
						case KXmlParser.END_TAG:
							if(parser.getName().equals("RasterMetaFile"))
								end = true;
							tag = parser.next();
							break;
							
						default: 
							tag = parser.next();
						}
					}
					if (!exists)
						return null;
				}
			}
		}catch(XmlPullParserException e){
			return null;
		}catch(FileNotFoundException ex){
			return null;
		}catch(IOException exc){
			return null;
		}
		
		return hist;
		
	}
	
	/**
	 * Obtiene el histograma desde el fichero RMF si tiene y calcula a partir de
	 * este el acumulado
	 * @return 
	 */
	public int[][] readFullAccumulatedHistogramFromRMF(){
		//TODO: implementar readFullAccumulatedHistogramFromRMF()
		this.readFullHistogramFromRMF();
		//.....
		return null;
	}
	
	/**
	 * Escribe el histograma en el fichero RMF de disco asociado al raster. Esta escritura
	 * solo se realiza si no se ha leido de RMF
	 */
	public void writeFullHistogramToRMF() throws IOException{
		File rmfFile = null;
		String grfName = null;
		tools = new HistogramPersistence();
		int[][] grfHistogram = null;
		
		//Cuando solo existe un GeoRasterFile en el Grid
		if (grid.getGeoRasterMultiFile().getFileCount() == 1)
			if (!isReadingFromRMF[0]){
				grfName = grid.getGeoRasterMultiFile().getFile(0).getName();
				int grfBands = grid.getGeoRasterMultiFile().getFile(0).getBandCount();
				tools.makeRMFHistogram(histogram, grfName, grfBands);				
			}
		
		//Si el Grid contiene mas de un GeoRasterFile
		if (grid.getGeoRasterMultiFile().getFileCount() > 1){
			for (int i = 0; i < grid.getGeoRasterMultiFile().getFileCount() ; i++){
				int grfBands = grid.getGeoRasterMultiFile().getFile(i).getBandCount();
				int grfCount = 0;
				grfName = grid.getGeoRasterMultiFile().getFile(i).getName();
				boolean isReading = false;
				grfHistogram = new int[grfBands][256];
				for (int j = 0 ; j < grid.getBandCount() ; j++){
					if (grid.getGeoRasterMultiFile().getBands().getBandStringList()[j].equals(grfName)){
						grfHistogram[grfCount] = histogram[j];
						grfCount++;
						//if(isReadingFromRMF[j])
							//isReading = true;
						}
				}
				if (!isReading)
					tools.makeRMFHistogram(grfHistogram, grfName, grfBands);
			}
		}
	}
	
	/**
	 * Calcula el histograma de la imagen completa.
	 */
	public synchronized void run(){
		while(true){
			//Asignamos las bandas que van a dibujarse
			int[] drawableBands = new int[grid.getBandCount()];
			for(int i = 0; i < grid.getBandCount(); i++)
				drawableBands[i] = i;
			grid.clearDrawableBand();
			grid.addDrawableBands(drawableBands);
			
			int heightArea = grid.getHeight();
			int widthArea = grid.getWidth();
			int poxInitX = 0;
			int lastY = 0;
			if(!calcFullHistogram){ //Calculo del histograma en el rea de interes
				heightArea = heightAOI;
				widthArea = widthAOI;
				poxInitX = xAOI;
				lastY = yAOI;
			}
			
			//Calculamos bloques de tamao BLOCK para no leer toda la imagen de golpe
			int nBlocks = (int)(heightArea / BLOCK);
			int lastBlock = heightArea - (nBlocks * BLOCK);
			int incrPercent = (int)(100 / (nBlocks + 1));
	
			switch(grid.getDataType()){
			case RasterBuf.TYPE_BYTE: byteHistogram(poxInitX, lastY, widthArea, nBlocks, lastBlock, incrPercent); break;
			case RasterBuf.TYPE_SHORT: shortHistogram(poxInitX, lastY, widthArea, nBlocks, lastBlock, incrPercent); break;
			case RasterBuf.TYPE_INT: intHistogram(poxInitX, lastY, widthArea, nBlocks, lastBlock, incrPercent); break;
			}
			percent = 100;
			
			//Si hemos calculado el histograma de la imagen completa lo salvamos a disco
			if(calcFullHistogram)
				try{
					writeFullHistogramToRMF();
				}catch(IOException ex){
					System.err.println("No se ha podido guardar el histograma");
				}
			grid.free();
			this.suspend();
		}
	}

	/**
	 * Calcula el histograma para tipo de datos byte
	 * @param nBlocks Nmero de bloques 
	 * @param lastBlock Tamao del ltimo bloque
	 * @param lastY posicin Y donde empieza el primer bloque 
	 */
	private void byteHistogram(int x, int lastY, int widthArea, int nBlocks, int lastBlock, int incrPercent){
		histogram = new int[grid.getBandCount()][Utilities.MAX_BYTE_BIT_VALUE + 1];
		for(int nBlock = 0; nBlock < nBlocks; nBlock++){
			grid.setAreaOfInterest(x, lastY, widthArea, BLOCK);
			RasterBuf raster = grid.getRasterBuf();
			for(int iBand = 0; iBand < grid.getBandCount(); iBand ++)
				for(int line = 0; line < BLOCK; line ++)
					for(int col = 0; col < widthArea; col ++)
						histogram[iBand][(raster.getElemByte(line, col, iBand) & 0xff)]++;
			lastY += BLOCK;
			percent += incrPercent;
		}
		grid.setAreaOfInterest(x, lastY, widthArea, lastBlock);
		RasterBuf raster = grid.getRasterBuf();
		for(int iBand = 0; iBand < grid.getBandCount(); iBand ++)
			for(int line = 0; line < lastBlock; line ++)
				for(int col = 0; col < widthArea; col ++)
					histogram[iBand][(raster.getElemByte(line, col, iBand) & 0xff)]++;	
		
		accumulateAllDataTypes(Utilities.MAX_BYTE_BIT_VALUE + 1);
	}
	
	/**
	 * Calcula el histograma para tipo de datos short
	 * @param nBlocks Nmero de bloques 
	 * @param lastBlock Tamao del ltimo bloque
	 * @param lastY posicin Y donde empieza el primer bloque 
	 */
	private void shortHistogram(int x, int lastY, int widthArea, int nBlocks, int lastBlock, int incrPercent){
		histogram = new int[grid.getBandCount()][Utilities.MAX_SHORT_BIT_VALUE + 1];
		for(int nBlock = 0; nBlock < nBlocks; nBlock++){
			grid.setAreaOfInterest(x, lastY, widthArea, BLOCK);
			RasterBuf raster = grid.getRasterBuf();
			for(int iBand = 0; iBand < grid.getBandCount(); iBand ++)
				for(int line = 0; line < BLOCK; line ++)
					for(int col = 0; col < widthArea; col ++)
						histogram[iBand][(raster.getElemShort(line, col, iBand) & 0xffff)]++;
			lastY += BLOCK;
			percent += incrPercent;
		}
		grid.setAreaOfInterest(x, lastY, widthArea, lastBlock);
		RasterBuf raster = grid.getRasterBuf();
		for(int iBand = 0; iBand < grid.getBandCount(); iBand ++)
			for(int line = 0; line < lastBlock; line ++)
				for(int col = 0; col < widthArea; col ++)
					histogram[iBand][(raster.getElemShort(line, col, iBand) & 0xffff)]++;	
		
		accumulateAllDataTypes(Utilities.MAX_SHORT_BIT_VALUE + 1);
	}
	
	/**
	 * Calcula el histograma para tipo de datos short
	 * @param nBlocks Nmero de bloques 
	 * @param lastBlock Tamao del ltimo bloque
	 * @param lastY posicin Y donde empieza el primer bloque 
	 */
	private void intHistogram(int x, int lastY, int widthArea, int nBlocks, int lastBlock, int incrPercent){
		histogram = new int[grid.getBandCount()][Utilities.MAX_SHORT_BIT_VALUE + 1];
		for(int nBlock = 0; nBlock < nBlocks; nBlock++){
			grid.setAreaOfInterest(x, lastY, widthArea, BLOCK);
			RasterBuf raster = grid.getRasterBuf();
			for(int iBand = 0; iBand < grid.getBandCount(); iBand ++)
				for(int line = 0; line < BLOCK; line ++)
					for(int col = 0; col < widthArea; col ++)
						histogram[iBand][(raster.getElemInt(line, col, iBand) & 0xffff)]++;
			lastY += BLOCK;
			percent += incrPercent;
		}
		grid.setAreaOfInterest(x, lastY, widthArea, lastBlock);
		RasterBuf raster = grid.getRasterBuf();
		for(int iBand = 0; iBand < grid.getBandCount(); iBand ++)
			for(int line = 0; line < lastBlock; line ++)
				for(int col = 0; col < widthArea; col ++)
					histogram[iBand][(raster.getElemInt(line, col, iBand) & 0xffff)]++;	
		
		accumulateAllDataTypes(Utilities.MAX_SHORT_BIT_VALUE + 1);
	}
	
	/**
	 * Calcula el histograma acumulado a partir del normal ya calculado. Esta llamada es usada para
	 * el histograma a partir de la fuente de datos. Adems de calcular el acumulado elimina las
	 * bandas que no se han pedido
	 * @param nValues Nmero de valores del histograma
	 */
	private void accumulateAllDataTypes(int nValues){
		//Acumulado, adems eliminamos bandas que no se han pedido
		accumulatedHistogram = new int[grid.getBandCount()][nValues];
		for(int iBand = 0; iBand < accumulatedHistogram.length; iBand++){
			if(showBands[iBand])
				accumulatedHistogram[iBand][0] = histogram[iBand][0];
			else
				histogram[iBand][0] = 0;
			for(int i = 1; i < accumulatedHistogram[iBand].length; i++){
				if(showBands[iBand])
					accumulatedHistogram[iBand][i] = histogram[iBand][i] + accumulatedHistogram[iBand][i - 1];
				else
					histogram[iBand][i] = 0;
			}
		}
		
	}
	
	/**
	 * Obtiene el porcentaje que lleva construido el histograma
	 * @return porcentaje de construccin
	 */
	public int getPercent() {
		return percent;
	}
	
	/**
	 * calcula el histograma de la imagen completa. En caso de que exista un fichero RMF con 
	 * el histograma calculado se leer de all.
	 */
	public void calcFullHistogram(boolean[] bands){
		percent = 0;
		showBands = bands;
		calcFullHistogram = true;
		histogram = null;
		histogram = readFullHistogramFromRMF();
		if(histogram == null){
			if(this.isAlive())
				resume();
			else
				start();
		}else
			percent = 100;
	}
	
	/**
	 * calcula el histograma acumulado de la imagen completa. En caso de que exista un fichero RMF con 
	 * el histograma calculado se leer de all.
	 */
	public void calcFullAccumulatedHistogram(boolean[] bands){
		percent = 0;
		showBands = bands;
		calcFullHistogram = true;
		histogram = null;
		histogram = readFullAccumulatedHistogramFromRMF();
		if(histogram == null){
			if(this.isAlive())
				resume();
			else
				start();
		}else{
			
			percent = 100;
		}
	}
	
	/**
	 * Calcula el histograma de la imagen correspondiente al rea de interes. Antes de llamar a esta funcin 
	 * hay que asignar el rea de interes
	 */
	public void calcHistogramFromLimetedArea(boolean[] bands){
		percent = 0;
		showBands = bands;
		calcFullHistogram = false;
		if(this.isAlive())
			resume();
		else
			start();
	}
	
	/**
	 * Calcula el histograma de la imagen correspondiente al rea de interes. Este rea de interes es
	 * es obtenida del extent del renderizador. 
	 */
	public boolean calcHistogramFromRenderizerLimetedArea(boolean[] bands){
		percent = 0;
		showBands = bands;
		calcFullHistogram = false;
		Extent fullExtent = pxRaster.getExtent();
		Extent wcArea = new Extent(pxRaster.getLastViewPort().getExtent());
		
		if(	wcArea.getMax().getX() < fullExtent.getMin().getX() ||
				wcArea.getMin().getX() > fullExtent.getMax().getX() ||	
				wcArea.getMax().getY() < fullExtent.getMin().getY() ||
				wcArea.getMin().getY() > fullExtent.getMax().getY() ){
			histogram = new int[grid.getBandCount()][Utilities.MAX_BYTE_BIT_VALUE + 1];
			accumulatedHistogram = new int[grid.getBandCount()][Utilities.MAX_BYTE_BIT_VALUE + 1];
			return false;
		}
		wcArea = Utilities.calculateAdjustedView(wcArea, fullExtent);
		
		Rectangle2D pxArea = Utilities.getPxRectFromMapRect(pxRaster.getFiles()[0].getExtent().toRectangle2D(), pxRaster.getFiles()[0].getWidth(), pxRaster.getFiles()[0].getHeight(), wcArea.toRectangle2D());
		setAreaOfInterest((int)pxArea.getMinX(), (int)pxArea.getMinY(), (int)Math.abs(pxArea.getMaxX() - pxArea.getMinX()), (int)Math.abs(pxArea.getMaxY() - pxArea.getMinY()));
		if(this.isAlive())
			resume();
		else
			start();
		return true;
	}
	
	/**
	 * Calcula el histograma de la vista y solo con los datos de visualizacin
	 * @return Histograma en forma de array bidimensional de enteros en el que el 
	 * el primer indice es el nmero de bandas y el segundo el histograma por banda
	 */
	public int[][] calcHistogramFromView(boolean[] bands){
		viewHistogram = new int[3][256];
		if(pxRaster != null){
			BufferedImage img = (BufferedImage)pxRaster.getGeoImage();
			if(img !=null)
				for(int col = 0; col < img.getHeight(); col++){
					for(int line = 0; line < img.getWidth(); line++){
						int px = ((BufferedImage) img).getRGB(line, col);
						int px3[] = {(px & 0xff0000) >> 16, (px & 0xff00) >> 8, (px & 0xff)};
						if(bands[0])
							viewHistogram[0][px3[0]] ++;  		//Histograma para la banda R
						if(bands[1])
							viewHistogram[1][px3[1]] ++;  		//Histograma para la banda G
						if(bands[2])
							viewHistogram[2][px3[2]] ++;  		//Histograma para la banda B
					}
				}
		}
		return viewHistogram;
	}
		
	/**
	 * Calcula el histograma de la vista acumulado y solo con los datos de visualizacin
	 * @return Histograma en forma de array bidimensional de enteros en el que el 
	 * el primer indice es el nmero de bandas y el segundo el histograma por banda
	 */
	public int[][] calcAccumulatedHistogramFromView(boolean[] bands){
		viewHistogram = calcHistogramFromView(bands);
		for(int iBand = 0; iBand < viewHistogram.length; iBand++){
			for(int i = 1; i < viewHistogram[iBand].length; i++){
				viewHistogram[iBand][i] += viewHistogram[iBand][i - 1];
			}
		}
		return viewHistogram;
	}
	
}
