/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.datastruct;

import java.util.ArrayList;


/**
 * Clase para estadisticas
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class Statistic {
    /**
     * Mnimo valor de pixel por banda
     */
    public double[] minBandValue = {
                                    Double.MAX_VALUE, Double.MAX_VALUE,
                                    Double.MAX_VALUE
                                };

    /**
     * Mximo valor de pixel por banda
     */
    public double[] maxBandValue = {
                                    Double.MIN_VALUE, Double.MIN_VALUE,
                                    Double.MIN_VALUE
                                };

    /**
     * Segundo valor ms pequeo de pixel por banda
     */
    public double[] secondMinBandValue = {
                                          Double.MAX_VALUE, Double.MAX_VALUE,
                                          Double.MAX_VALUE
                                      };

    /**
     * Segundo valor ms grande de pixel por banda
     */
    public double[] secondMaxBandValue = {
                                          Double.MIN_VALUE, Double.MIN_VALUE,
                                          Double.MIN_VALUE
                                      };

    /**
     * Historial de valores para una imagen. Esto se usa cuando se solicita varias
     * veces para una misma imagen solo tenga que calcular los valores una vez
     */
    public ArrayList history = new ArrayList();

    /**
     * Porcentaje de recorte de colas
     */
    public double tailPercent = 0D;

    /**
     * Constructor
     */
    public Statistic() {
        super();
    }
    
    /**
     * Calculo de estadsticas a partir de un histograma. El resultado de la funcin es un array
     * bidimensional donde el primer ndice inndica la estadistica y el segundo el nmero de banda.
     * 
     * <UL>
     * <LI>mnimo</LI>
     * <LI>mximo</LI>
     * <LI>media</LI>
     * <LI>mediana</LI>
     * <LI>Nmero de pixels</LI>
     * </UL>
     * @param histogram
     * @param bandas solicitadas. Cada elemento del vector representa una banda. Si est a true se calcula la 
     * estadistica para esa banda y si est a false no se calcular.
     * @return
     */
    public static long[][] getBasicStatsFromHistogram(int[][] histogram, boolean[] bands){
    	if(histogram == null)
    		return null;
    	return getBasicStatsFromHistogram(histogram, 0, histogram[0].length - 1, bands);
    }
    /**
     * Calculo de estadsticas a partir de un histograma. El resultado de la funcin es un array
     * bidimensional donde el primer ndice inndica la estadistica y el segundo el nmero de banda.
     * 
     * <UL>
     * <LI>mnimo</LI>
     * <LI>mximo</LI>
     * <LI>media</LI>
     * <LI>mediana</LI>
     * <LI>Nmero de pixels</LI>
     * </UL>
     * @param histogram
     * @param beginPos Posicin de inicio del histograma para contabilizar estadisticas
     * @param endPos Posicin de fin del histograma para contabilizar estadisticas
     * @param bandas solicitadas. Cada elemento del vector representa una banda. Si est a true se calcula la 
     * estadistica para esa banda y si est a false no se calcular.
     * @return
     */
    public static long[][] getBasicStatsFromHistogram(int[][] histogram, int beginPos, int endPos, boolean[] bands){
    	if(histogram == null)
    		return null;
    	
    	//Contamos el nmero de bandas para las cuales se calcula la estadstica
    	int bandCount = 0;
    	for(int iBand = 0; iBand < bands.length; iBand ++)
    		if(bands[iBand])
    			bandCount ++;
    	
    	int values = 5;
    	long[][] res = new long[values][];
    	
    	long[] min = new long[bandCount];//Mnimo
    	long[] max = new long[bandCount];//Mximo
    	for(int iBand = 0; iBand < bandCount; iBand ++){
    		max[iBand] = beginPos;
    		min[iBand] = endPos;
    	}
    	long[] average = new long[bandCount]; //Valor de pixel medio (Media)
    	long[] middle = new long[bandCount]; //Mediana
    	long[] nPixelsBand = new long[bandCount];//Nmero de pixels por banda
    	    	    	    	
    	int showBandCounter = 0;  //Contador de bandas de las que hay calcular la estadistica
    	for(int iBand = 0; iBand < histogram.length; iBand ++){
    		if(bands[iBand]){
	    		int pixels = 0; //Nmero de valores por banda (entre 0 y 255)
				for(int i = beginPos; i <= endPos; i ++){
					
					//Calculo del mnimo
					if(histogram[iBand][i] != 0 && i < min[showBandCounter])
						min[showBandCounter] = i;
					
					//Calculo del mximo										
					if(histogram[iBand][i] != 0 && i > max[showBandCounter])
						max[showBandCounter] = i;
					
					//Calculo del nmero de pixeles
					nPixelsBand[showBandCounter] += (long)histogram[iBand][i];
					
					if(histogram[iBand][i] != 0)
						pixels ++;
					
					average[showBandCounter] += histogram[iBand][i] * i;
				}
				//Calculo de la media
				try{
					average[showBandCounter] /= nPixelsBand[showBandCounter];
				}catch(ArithmeticException exc){
					average[showBandCounter] = 0;
				}
				
				//Calculo de mediana
				long middlePos = nPixelsBand[showBandCounter] >> 1;
				int aux = 0;
				int i = beginPos;
				for(i = beginPos; aux < middlePos; i++)
					aux += histogram[iBand][i];
				middle[showBandCounter] = i - 1;
				
				showBandCounter ++;
    		}
    	}
     
    	res[0] = min;
    	res[1] = max;
    	res[2] = average;
    	res[3] = middle;
    	res[4] = nPixelsBand;
    	return res;	
    }
    
    /**
     * Clase que contiene los valores calculados para una imagen.
     * Esto se usa cuando se solicita varias veces para una misma imagen solo
     * tenga que calcular los valores una vez.
     * @author Nacho Brodin (brodin_ign@gva.es)
     */
    public class History {
        public String file = null;
        public double[] min = null;
        public double[] max = null;
        public double[] secMin = null;
        public double[] secMax = null;

        public History(String file, double[] min, double[] max, double[] secMin,
                       double[] secMax) {
            this.file = file;
            this.min = (double[]) min.clone();
            this.max = (double[]) max.clone();
            this.secMin = (double[]) secMin.clone();
            this.secMax = (double[]) secMax.clone();
        }
    }
}
