package com.hardcode.gdbms.engine.data.edition;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.InnerDBUtils;
import com.hardcode.gdbms.engine.data.db.DBDataSourceAdapter;
import com.hardcode.gdbms.engine.data.db.DBTableSourceInfo;
import com.hardcode.gdbms.engine.data.driver.AlphanumericDBDriver;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueCollection;
import com.hardcode.gdbms.engine.values.ValueWriter;


/**
 * DataSource with write capabilities over a internal database source.
 *
 * @author Fernando Gonzlez Corts
 */
public class InternalBuffer extends DBDataSourceAdapter {
    private Statement s;
    private ResultSet rs;
    private String[] pkNames;
    private String[] fieldNames;
    private int[] pkIndices;
    private int lastInserted = 0;
    private int rowCount;
    private boolean refreshNeeded = true;

    /**
     * Creates a new InternalBufferDataWare object.
     *
     * @param dsf DataSourceFactory
     * @param pkNames primary key field names
     * @param names field names
     * @param types field types
     *
     * @throws DriverException If cannot create a new InternalBuffer
     */
    public InternalBuffer(DataSourceFactory dsf, String[] pkNames,
	        String[] names, int[] types) throws DriverException {
    	try {
    	
	        this.dsf = dsf;
	
	        //Add a secuential number
	        String databaseName = dsf.getTempFile();
	        String tableName = "gdbms";
	
	        
	            InnerDBUtils.execute(databaseName,
	                InnerDBUtils.getCreateStatementWithAutonumeric(tableName,
	                    names, types));
	            InnerDBUtils.execute(databaseName,
	                InnerDBUtils.getPKIndexStatement(tableName, pkNames));
	        //Cache of metadata
	        this.pkNames = pkNames;
	        this.fieldNames = names;
	
	        //DataSource configuration
	        DBTableSourceInfo di = new DBTableSourceInfo();
	        di.name = tableName;
	        di.dbName = databaseName;
	        di.tableName = tableName;
	        di.driverName = "GDBMS HSQLDB driver";
	        setSourceInfo(di);
			setDriver((AlphanumericDBDriver) dsf.getDriverManager().getDriver(di.driverName));
			rowCount = 0;
	    } catch (SQLException e) {
	        throw new DriverException(e);
		} catch (DriverLoadException e) {
	        throw new DriverException(e);
	    }
        
    }

    /**
     * @see com.hardcode.gdbms.engine.data.edition.DataWare#deleteRow(long)
     */
    public void deleteRow(long rowId) throws DriverException {
        //Get the pk values
        ValueCollection pks = getPKValue(rowId);
        String[] names = getFieldNames();

        String sql = InnerDBUtils.createDeleteStatement(pks.getValues(), names, tableName, ValueWriter.internalValueWriter);

        try {
            s.execute(sql.toString());
            rowCount--;
            refreshNeeded = true;
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Inserts a row with data
     *
     * @param values values of the data to be inserted
     *
     * @return index where the row is inserted
     *
     * @throws DriverException if the operation fails
     */
    public long insertFilledRow(Value[] values) throws DriverException {
        /*
         * the last field is the automuneric gdbmsindex
         */
        String[] names = new String[getFieldCount() - 1];
        System.arraycopy(getFieldNames(), 0, names, 0, names.length);

        String sql = InnerDBUtils.createInsertStatement(tableName, values, names, ValueWriter.internalValueWriter);

        try {
            s.execute(sql);
            rowCount++;
            refreshNeeded = true;

            return rowCount - 1;
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Inserts a new row with only the pk values
     *
     * @param pk value of the PK
     *
     * @return index where the row is inserted
     *
     * @throws DriverException If the operation fails
     */
    public long insertRow(ValueCollection pk) throws DriverException {
        Value[] pks = pk.getValues();

        String sql = InnerDBUtils.createInsertStatement(tableName, pks, pkNames, ValueWriter.internalValueWriter);

        try {
            s.execute(sql);
            rowCount++;
            refreshNeeded = true;

            return rowCount - 1;
        } catch (SQLException e1) {
            throw new DriverException(e1);
        }
    }

    /**
     * Set the modified row at the specified index
     *
     * @param row index of the row to update
     * @param modifiedRow Value array with the update
     *
     * @throws DriverException if the update fails
     */
    public void setRow(long row, Value[] modifiedRow) throws DriverException {
        Value[] pks = new Value[getPkIndices().length];

        for (int i = 0; i < pks.length; i++) {
            pks[i] = getFieldValue(row, getPkIndices()[i]);
        }

        String sql = InnerDBUtils.createUpdateStatement(tableName, pks,
                pkNames, fieldNames, modifiedRow, ValueWriter.internalValueWriter);

        try {
            s.execute(sql);
            refreshNeeded = true;
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * @see com.hardcode.gdbms.engine.data.edition.DataWare#setFieldValue(long,
     *      int, com.hardcode.gdbms.engine.values.Value)
     */
    public void setFieldValue(long row, int modifiedField, Value modifiedValue)
        throws DriverException {
        ValueCollection pk = getPKValue(row);
        Value[] pks = getPKValue(row).getValues();
        String[] names = new String[pk.getValueCount()];

        for (int i = 0; i < pk.getValueCount(); i++) {
            names[i] = getPKName(i);
        }

        String sql = InnerDBUtils.createUpdateStatement(tableName, pks, names,
                new String[] { getFieldName(modifiedField) },
                new Value[] { modifiedValue }, ValueWriter.internalValueWriter);

        try {
            s.execute(sql);
            refreshNeeded = true;
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#start()
     */
    public void start() throws DriverException {
        try {
            con = this.getConnection();

            String localSql = sql + " ORDER BY GDBMSINDEX";
            ((AlphanumericDBDriver) driver).open(con, localSql);
            s = con.createStatement();
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#stop()
     */
    public void stop() throws DriverException {
        try {
            s.close();
            con.close();
            con = null;
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Gets a int array containing the field index of the primary key fields
     *
     * @return int[]
     *
     * @throws DriverException if the operation fails
     */
    private int[] getPkIndices() throws DriverException {
        if (pkIndices == null) {
            pkIndices = new int[pkNames.length];

            for (int i = 0; i < pkIndices.length; i++) {
                this.pkIndices[i] = getFieldIndexByName(pkNames[i]);
            }
        }

        return pkIndices;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPKValue(long)
     */
    public ValueCollection getPKValue(long rowIndex) throws DriverException {
        refresh();

        return super.getPKValue(rowIndex);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getFieldValue(long,
     *      int)
     */
    public Value getFieldValue(long rowIndex, int fieldId)
        throws DriverException {
        refresh();

        return super.getFieldValue(rowIndex, fieldId);
    }

    /**
     * Rereads the information if has been modified
     * since last refresh
     *
     * @throws DriverException If the operation fails
     */
    private void refresh() throws DriverException {
        if (refreshNeeded == true) {
            stop();
            start();
            refreshNeeded = false;
        }
    }
}
