package com.hardcode.gdbms.engine.instruction;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.hardcode.gdbms.parser.Node;


/**
 * Wrapper sobre las expresiones unarias en el arbol sintctico de entrada
 *
 * @author Fernando Gonzlez Corts
 */
public class UnaryExprAdapter extends AbstractExpression implements Expression {
	private boolean signChange = false;

	/**
	 * Evalua expresin invocando el mtodo adecuado en funcin del tipo de
	 * expresion (suma, producto, ...) de los objetos Value de la expresion,
	 * de las subexpresiones y de los objetos Field
	 *
	 * @param row Fila en la que se evala la expresin, en este caso no es
	 * 		  necesario, pero las subexpresiones sobre las que se opera pueden
	 * 		  ser campos de una tabla, en cuyo caso si es necesario
	 *
	 * @return Objeto Value resultado de la operacin propia de la expresin
	 * 		   representada por el nodo sobre el cual ste objeto es adaptador
	 *
	 * @throws SemanticException Si se produce un error semntico
	 * @throws DriverException Si se produce un error de I/O
	 */
	public Value evaluate(long row) throws EvaluationException {
		Value ret = null;

		Adapter[] terms = (Adapter[]) getChilds();

		if (terms.length > 0) {
			ret = ((Expression) terms[0]).evaluateExpression(row);

			for (int i = 1; i < terms.length; i++) {
				try {
                    ret = ret.suma(((Expression) terms[i]).evaluateExpression(row));
                } catch (IncompatibleTypesException e) {
                    throw new EvaluationException(e);
                }
			}
		}

		if (signChange) {
			Value menosUno = ValueFactory.createValue(-1);
			try {
                ret = ret.producto(menosUno);
            } catch (IncompatibleTypesException e) {
                throw new EvaluationException(e);
            }
		}

		return ret;
	}

	/**
	 * @see com.hardcode.gdbms.engine.instruction.Expression#getFieldName()
	 */
	public String getFieldName() {
		Adapter[] expr = (Adapter[]) getChilds();

		if (expr.length != 1) {
			return null;
		} else {
			return ((Expression) expr[0]).getFieldName();
		}
	}

	/**
	 * @see com.hardcode.gdbms.engine.instruction.Expression#isLiteral()
	 */
	public boolean isLiteral() {
		return ((Expression) getChilds()[0]).isLiteral();
	}

	/**
	 * @see com.hardcode.gdbms.engine.instruction.Expression#simplify()
	 */
	public void simplify() {
	}

	/**
	 * @see com.hardcode.gdbms.engine.instruction.Adapter#setEntity(com.hardcode.gdbms.parser.Node)
	 */
	public void setEntity(Node o) {
		super.setEntity(o);

		String text = Utilities.getText(getEntity());

		if (text.startsWith("-")) {
			signChange = true;
		}
	}
	
    /**
     * @see com.hardcode.gdbms.engine.instruction.Expression#isAggregated()
     */
    public boolean isAggregated() {
		Adapter[] expr = (Adapter[]) getChilds();

		if (expr.length != 1) {
			return false;
		} else {
			return ((Expression) expr[0]).isAggregated();
		}
    }
	
}
