package org.agil.core.spatialindex;
import java.util.*;

/**
 *  A Leaf node. Contains pointers to the real data. <p>
 *
 *  Created: Tue May 18 13:04:08 1999 <p>
 *
 *
 *
 *@author     Marios Hadjieleftheriou
 *@version    2
 */
public class Leaf extends AbstractNode {
	/**
	 *  Constructor for the Leaf object
	 *
	 *@param  tree        Description of Parameter
	 *@param  parent      Description of Parameter
	 *@param  pageNumber  Description of Parameter
	 */
	protected Leaf(RTree tree, long parent, long pageNumber) {
		// Leaf nodes belong by default to level 0.
		super(tree, parent, pageNumber, 0);
	}


	/**
	 *  Constructor for the Leaf object
	 *
	 *@param  tree    Description of Parameter
	 *@param  parent  Description of Parameter
	 */
	protected Leaf(RTree tree, long parent) {
		// Leaf nodes belong by default to level 0.
		super(tree, parent, -1, 0);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  h  Description of Parameter
	 *@return    Description of the Returned Value
	 */
	protected Leaf chooseLeaf(HyperCube h) {
		return this;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  h  Description of Parameter
	 *@return    Description of the Returned Value
	 */
	protected Leaf findLeaf(HyperCube h) {
		for (int i = 0; i < usedSpace; i++) {
			if (data[i].equals(h)) {
				return this;
			}
		}
		return null;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  h    Description of Parameter
	 *@param  pos  Description of Parameter
	 *@return      Description of the Returned Value
	 */
	protected Leaf findLeaf(HyperCube h, long pos) {
		for (int i = 0; i < usedSpace; i++) {
			if (data[i].equals(h)) {
				if (branches[i] == pos) {
					return this;
				}
			}
		}
		return null;
	}


	/**
	 *  Inserts a new HyperCube into the Leaf. If there is space left, then the
	 *  insertion is straightforward and adjustTree must be called. Otherwise, a
	 *  split must occur and two new leaves must be created.
	 *
	 *@param  h    The HyperCube that represents the mbb of the data.
	 *@param  oid  Object identifier of the data.
	 *@return      True if a split occurred, false otherwise.
	 */
	protected boolean insert(HyperCube h, long oid) {
		//System.out.println("UsedSpace:" + usedSpace + " NodeCapacity:" + tree.getNodeCapacity());
		if (usedSpace < tree.getNodeCapacity()) {
			addData(h, oid);
			tree.file.writeNode(this);
			Index p = (Index) getParent();
			if (p != null) {
				p.adjustTree(this, null);
			}
			return false;
		}
		else {
			Leaf[] a = splitLeaf(h, oid);
			Leaf l = a[0];
			Leaf ll = a[1];
			if (isRoot()) {
				// root is full, so we must split it. From now on root will be
				// an Index and not a Leaf.
				l.parent = 0;
				l.pageNumber = -1;
				ll.parent = 0;
				ll.pageNumber = -1;
				tree.file.writeNode(l);
				tree.file.writeNode(ll);
				// create the new root node. It has pageNumber 0 and level 1.
				Index r = new Index(tree, RTree.NIL, 0, 1);
				r.addData(l.nodeMbb, l.pageNumber);
				r.addData(ll.nodeMbb, ll.pageNumber);
				tree.file.writeNode(r);
			}
			else {
				// use old page number for left child, a new page number for the right child.
				l.pageNumber = pageNumber;
				ll.pageNumber = -1;
				tree.file.writeNode(l);
				tree.file.writeNode(ll);
				Index p = (Index) getParent();
				p.adjustTree(l, ll);
			}
			tree.stats.splits++;
			return true;
		}
	}


	/**
	 *  Deletes an entry from the leaf. [A. Guttman 'R-trees a dynamic index
	 *  structure for spatial searching'] 3.3. Deletion
	 *
	 *@param  h  The hypercube that corresponds to the entry to be deleted.
	 *@return    The object id of the deleted entry.
	 */
	protected long delete(HyperCube h) {
		for (int i = 0; i < usedSpace; i++) {
			if (data[i].equals(h)) {
				long oid = branches[i];
				deleteData(i);
				tree.file.writeNode(this);
				Stack s = new Stack();
				condenseTree(s);
				// re-insert eliminated leaves.
				while (!s.empty()) {
					AbstractNode n = (AbstractNode) s.pop();
					for (int j = 0; j < n.usedSpace; j++) {
						if (n.isLeaf()) {
							tree.insert(n.data[j], n.branches[j]);
						}
						else {
							s.push(((Index) n).getChild(j));
						}
						//ALEX - lo que modifico
						//tree.file.deletePage(n.pageNumber);
					}
					//ALEX - lo que habia
					if (!n.isLeaf()) {
						tree.file.deletePage(n.pageNumber);
					}
				}
				return oid;
			}
		}
		return RTree.NIL;
	}


	/**
	 *  Deletes an entry from the leaf if the Brach is equals to 'pos' [A.
	 *  Guttman 'R-trees a dynamic index structure for spatial searching'] 3.3.
	 *  Deletion
	 *
	 *@param  h    The hypercube that corresponds to the entry to be deleted.
	 *@param  pos  The branch that corresponds to the entry to be deleted.
	 *@return      The object id of the deleted entry.
	 */
	protected long delete(HyperCube h, long pos) {
		for (int i = 0; i < usedSpace; i++) {
			if (data[i].equals(h) && branches[i] == pos) {
				long oid = branches[i];
				deleteData(i);
				tree.file.writeNode(this);
				Stack s = new Stack();
				condenseTree(s);
				// re-insert eliminated leaves.
				while (!s.empty()) {
					AbstractNode n = (AbstractNode) s.pop();
					for (int j = 0; j < n.usedSpace; j++) {
						if (n.isLeaf()) {
							tree.insert(n.data[j], n.branches[j]);
						}
						else {
							s.push(((Index) n).getChild(j));
						}
						//ALEX - esto es lo que he modificado
						//tree.file.deletePage(n.pageNumber);
					}
					//ALEX - esto es lo que habia
					if (!n.isLeaf()) {
						tree.file.deletePage(n.pageNumber);
					}
				}
				return oid;
			}
		}
		return RTree.NIL;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  h    Description of Parameter
	 *@param  oid  Description of Parameter
	 *@return      Description of the Returned Value
	 */
	private Leaf[] splitLeaf(HyperCube h, long oid) {
		int[][] group = null;
		switch (tree.getTreeType()) {
			case RTree.RTREE_LINEAR:
			case RTree.RTREE_QUADRATIC:
				group = rtreeSplit((HyperCube) h.clone(), oid);
				break;
			case RTree.RSTAR:
				break;
			default:
				throw new IllegalStateException("Invalid tree type.");
		}
		//System.out.println("Haciendo SplitLeaf");
		Leaf l = new Leaf(tree, parent);
		Leaf ll = new Leaf(tree, parent);
		int[] g1 = group[0];
		int[] g2 = group[1];
		for (int i = 0; i < g1.length; i++) {
			l.addData(data[g1[i]], branches[g1[i]]);
		}
		for (int i = 0; i < g2.length; i++) {
			ll.addData(data[g2[i]], branches[g2[i]]);
		}
		return new Leaf[]{l, ll};
	}
}
// Leaf
