package org.agil.core.spatialindex;

import java.io.IOException;

/**
 *  Abstract class for all classes implementing a storage manager for the
 *  RTree. Every node should be stored in a unique page. The root node is
 *  always stored in page 0. The storage manager should have the control over
 *  the page numbers where each node should be stored. <p>
 *
 *  Created: Tue May 18 16:24:00 1999 <p>
 *
 *
 *
 *@author     Marios Hadjieleftheriou
 *@version    2
 */
public abstract class PageFile implements Cloneable {
	/**
	 *  Description of the Field
	 */
	protected RTree tree = null;
	/**
	 *  Dimension of data inserted into the tree.
	 */
	protected int dimension = -1;
	/**
	 *  fillFactor specifies minimum number of entries present in each node. It
	 *  must be a float between 0 and 0.5.
	 */
	protected float fillFactor = -1;
	/**
	 *  Maximum node capacity. Each node will be able to hold at most
	 *  nodeCapacity entries.
	 */
	protected int nodeCapacity = -1;
	/**
	 *  The page size needed in bytes to store a full node. Calculated using the
	 *  following formula: [nodeCapacity * (sizeof(HyperCube) + sizeof(Branch))]
	 *  + parent + level + usedSpace = {nodeCapacity * [(2 * dimension *
	 *  sizeof(float)) + sizeof(long)]} + sizeof(long) + sizeof(int) +
	 *  sizeof(int)
	 */
	protected int pageSize = -1;
	/**
	 *  The tree Type
	 */
	protected int treeType = -1;


	/**
	 *  Returns the node stored in the requested page.
	 *
	 *@param  page                Description of Parameter
	 *@return                     Description of the Returned Value
	 *@exception  PageFaultError  Description of Exception
	 */
	protected abstract AbstractNode readNode(long page) throws PageFaultError;


	/**
	 *  Writes the node into the first available page and returns that page.
	 *
	 *@param  o                   Description of Parameter
	 *@return                     Description of the Returned Value
	 *@exception  PageFaultError  Description of Exception
	 */
	protected abstract long writeNode(AbstractNode o) throws PageFaultError;


	/**
	 *  Marks a specific page as empty.
	 *
	 *@param  page                Description of Parameter
	 *@return                     Description of the Returned Value
	 *@exception  PageFaultError  Description of Exception
	 */
	protected abstract AbstractNode deletePage(long page) throws PageFaultError;


	/**
	 *  Clone de la clase
	 *
	 *@return    Description of the Returned Value
	 */
	protected abstract Object clone();


	/**
	 *  Description of the Method
	 *
	 *@param  tree        Description of Parameter
	 *@param  dimension   Description of Parameter
	 *@param  fillFactor  Description of Parameter
	 *@param  capacity    Description of Parameter
	 */
	protected void initialize(RTree tree, int dimension, float fillFactor, int capacity) {
		this.dimension = dimension;
		this.fillFactor = fillFactor;
		this.nodeCapacity = capacity;
		this.tree = tree;
		this.pageSize = capacity * (8 * dimension + 8) + 16;
	}


	/**
	 *  Description of the Method
	 *
	 *@exception  IOException  Description of Exception
	 */
	protected abstract void close() throws IOException;


	/**
	 *  Overrides the finalize method to dispose of system resources or to
	 *  perform other cleanup when the PageFile object is garbage collected.
	 *
	 *@exception  Throwable  Description of Exception
	 */
	protected void finalize() throws Throwable {
		super.finalize();
	}
}
// PageFile
